// Copyright (c) 2012 ARM Ltd
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

// ---------------------------------------------------------------------------
// This script runs in the content process and is loaded once for each page
// context. In effect, it loads in every tab. This script has the following
// responsibilities:
//
//    * Provide an annotation API to content JavaScript.
//
//    * Pass annotations as messages to the ARMGator object in the main
//      process. (Android only allows the main process to write to the local
//      file system.)
// ---------------------------------------------------------------------------

function Gator() {}
// This is the annotation function which will be called from content code. It
// can be accessed from content JavaScript as "window.Gator.annotate".
Gator.prototype.annotate = function annotate(message, colour) {
    let payload = { "message" : message };
    if ((colour !== undefined) && (colour.red !== undefined) && (colour.green !== undefined) && (colour.blue !== undefined)) {
        payload.colour = {
            "red"   : colour.red,
            "green" : colour.green,
            "blue"  : colour.blue
        };
    }
    // Use of synchronous messages is usually discouraged, particularly as the
    // recipient in this case will also perform a synchronous file access, but
    // it is necessary here in order to keep the annotation in sync with the
    // profile shown by Streamline.
    sendSyncMessage("ARMGator:Annotate", payload);
}
// Adding properties to the __exposedProps__ arrays exposes them to content
// code. The 'toString' property is necessary so that content JavaScript code
// can test for the presence of 'window.Gator', for feature-detection.
Gator.prototype.__exposedProps__ = {
    "toString" : "r",
    "annotate" : "r"
};
Gator.prototype.annotate.__exposedProps__ = {
    "toString" : "r",
};



function GatorManager() {
    // Register a "DOMWindowCreated" listener so the extension can attach a
    // Gator object to every DOM window in this frame.
    addEventListener("DOMWindowCreated", this, false);
}
GatorManager.prototype = {
    // Handle the DOMWindowCreated event.
    "handleEvent": function handleEvent(aEvent) {
        // Attach an instance of the Gator object to the DOM window's
        // defaultView. This will be visible to content JavaScript as
        // window.Gator.
        aEvent.target.defaultView.wrappedJSObject.Gator = new Gator();
    }
};
new GatorManager();
